/*
 * File:   tms1100.c
 * Author: seanriddle
 *
 * Feb 16, 2014
 */

//TMS-1100 emulator for PIC 18F46K22
//uses Paul Robson's TMS-1100 emulator code: http://mbmicrovision.blogspot.com/
// O outputs on A7:0
// R outputs on E2:0+D7:0
// K inputs  on B3:0
// /MCLR on E3 (reset), TX/RX on C6/C7 (debugging), PGC/PGD on B6/B7 (programming)
// unused: B5:4, C5:0 (C1:0 used to show timing)

//set to 1 to turn on a bunch of debug info
#define TDEBUG 0

#define _XTAL_FREQ 64000000

#pragma config FOSC=INTIO67,PLLCFG=OFF,PRICLKEN=ON,FCMEN=OFF,IESO=OFF
#pragma config PWRTEN=OFF,BOREN=OFF,BORV=285
#pragma config WDTEN=OFF,WDTPS=32768
#pragma config CCP2MX=PORTC1,PBADEN=OFF,CCP3MX=PORTB5,HFOFST=ON,T3CMX=PORTC0,P2BMX=PORTD2,MCLRE=EXTMCLR
#pragma config STVREN=ON,LVP=OFF,XINST=OFF,DEBUG=OFF
#pragma config CP0=OFF,CP1=OFF,CP2=OFF,CP3=OFF,CPB=OFF,CPD=OFF
#pragma config WRT0=OFF,WRT1=OFF,WRT2=OFF,WRT3=OFF,WRTC=OFF,WRTB=OFF,WRTD=OFF
#pragma config EBTR0=OFF,EBTR1=OFF,EBTR2=OFF,EBTR3=OFF,EBTRB=OFF

#define USE_OR_MASKS

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <xc.h>
#include <plib/usart.h>
#include "tms1100.h"
#include "code11.h"

void main(void) {
    TRISA=0;    //O outputs
    TRISB=0x0f; //K inputs
    TRISC=0xc0; //serial port
    TRISD=0;    //R outputs low bits
    TRISE=0;    //R outputs high bits

    LATA=0;
    LATC=0;
    LATD=0;
    LATE=0;

    //no analog or comparators
    ANSELA=0;
    ANSELB=0;
    ANSELC=0;
    ANSELD=0;
    ANSELE=0;
    ADCON0=0;
    CM1CON0=0;
    CM2CON0=0;

    //set up OSC and PLL, wait for PLL
    OSCCON=0x70;
    PLLEN=1;
    while(PLLRDY==0)
        ;

#if TDEBUG
#include "debugstuff.h"
#endif

    tms1100reset();

    TMR1H=INST_TIMEOUT_H;			// preset timer so overflow happens at TMS inst rate
    TMR1L=INST_TIMEOUT_L;
    TMR1IF=0;					// clear tmr1if
    T1CON=0x01;					// start timer 1, fOSC/4, 1:1 PS,16 bit r/w = 64MHz/4=16MHz

    while(1) {
        LATC^=1;                                //toggle C0 each instruction

        //fetch inst
        opcode.all=rom[ca][pa][lsfr[pc]];

#if TDEBUG
#include "debugstuff2.h"
#endif

        //increment program counter
        pc=(pc+1)&0x3f;

        //update status register
        lastStatus=status;
        status=1;

        //execute instruction
        #include "opcodes11.h"

        //kill time until next instruction cycle
        LATC|=0x02;                             // set C1 while spinning
        while(TMR1IF==0)                        // wait until timer overflows
            ;
        LATC&=0xfd;                             // clear C1
        T1CON=0x00;                             // stop timer 1
        TMR1H=INST_TIMEOUT_H;			// preset timer so overflow happens at TMS inst rate
        TMR1L=INST_TIMEOUT_L;
        T1CON=0x01;                             // start timer 1
        TMR1IF=0;                               // clear overflow flag
    }
}
